@@ -1,3 +1,4 @@
+;; -*-mode: emacs-lisp; tab-width: 8; indent-tabs-mode: t -*-
 ;; mu4e-compose.el -- part of mu4e, the mu mail user agent for emacs
 ;;
 ;; Copyright (C) 2011-2012 Dirk-Jan C. Binnema
@@ -110,7 +111,7 @@ for example:
                    'delete 'sent)))
 
 The various `message-' functions from `message-mode' are available
-for quering the message information."
+for querying the message information."
     :type '(choice (const :tag "move message to mu4e-sent-folder" sent)
 		 (const :tag "move message to mu4e-trash-folder" trash)
 		 (const :tag "delete message" delete))
@@ -224,42 +225,54 @@ appear on disk."
       ;; hide some headers again
       (mu4e~compose-hide-headers)
       (set-buffer-modified-p nil)
+      (mu4e-message "Saved (%d lines)" (count-lines (point-min) (point-max)))
       ;; update the file on disk -- ie., without the separator
       (mu4e~proc-add (buffer-file-name) mu4e~draft-drafts-folder)) nil t))
 
 
-(defun mu4e~compose-find-completion-style (some-style)
-  "Find completion style SOME-STYLE in completion-styles-alist, or return nil."
-  (find-if (lambda (style) (eq some-style (car style)))
-    completion-styles-alist))
-
-(defconst mu4e~completion-cycle-treshold 5
-  "mu4e value for `completion-cycle-treshold'.")
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; address completion; inspired by org-contacts.el and
+;; https://github.com/nordlow/elisp/blob/master/mine/completion-styles-cycle.el
 
+(defun mu4e~compose-complete-contact (&optional start)
+  "Complete the text at START with a contact.
+Ie. either 'name <email>' or 'email')."
+  (interactive)
+  (let ((mail-abbrev-mode-regexp mu4e~compose-address-fields-regexp)
+	 (eoh ;; end-of-headers
+	   (save-excursion
+	     (goto-char (point-min))
+	     (search-forward-regexp mail-header-separator nil t))))
+    ;; try to complete only when we're in the headers area,
+    ;; looking  at an address field.
+    (when (and eoh (> eoh (point)) (mail-abbrev-in-expansion-header-p))
+      (let* ((end (point))
+	      (start
+		(or start
+		  (save-excursion
+		    (re-search-backward "\\(\\`\\|[\n:,]\\)[ \t]*")
+		    (goto-char (match-end 0))
+		    (point)))))
+	(list start end mu4e~contacts-for-completion))))) 
+ 
 (defun mu4e~compose-setup-completion ()
-  "Set up autocompletion of addresses."
-  (let ((compstyle
-	  (or (mu4e~compose-find-completion-style 'substring)
-	    (mu4e~compose-find-completion-style 'partial-completion))))
-    ;; emacs-24+ has 'substring, otherwise we try partial-completion, otherwise
-    ;; we leave it at the default
-    (when compstyle
-      (make-local-variable 'completion-styles)
-      (add-to-list 'completion-styles (car compstyle)))
-    (when (boundp 'completion-cycle-threshold)
-      (make-local-variable 'completion-cycle-threshold)
-      (setq completion-cycle-threshold mu4e~completion-cycle-treshold))
-    ;; disable org-contacts support, since it prevents our autocompletion
-    (make-local-variable 'completion-at-point-functions)
-    (remove 'org-contacts-message-complete-function completion-at-point-functions)
-    (add-to-list 'completion-at-point-functions 'mu4e~compose-complete-contact)
-    ;; needed for emacs 23...
-    (when (= emacs-major-version 23)
-      (make-local-variable 'message-completion-alist)
-      (setq message-completion-alist
-	(cons
-	  (cons mu4e~compose-address-fields-regexp 'completion-at-point)
-	  message-completion-alist)))))
+  "Set up auto-completion of addresses."
+  (set (make-local-variable 'completion-ignore-case) t)
+  (set (make-local-variable 'completion-cycle-threshold) 7)
+  (add-to-list (make-local-variable 'completion-styles) 'substring t)
+  (add-hook 'completion-at-point-functions
+    'mu4e~compose-complete-contact nil t))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+(defvar mu4e-compose-mode-map nil
+  "Keymap for \"*mu4e-compose*\" buffers.")
+(unless mu4e-compose-mode-map
+  (setq mu4e-compose-mode-map
+    (let ((map (make-sparse-keymap)))
+      (define-key map (kbd "C-S-u")   'mu4e-update-mail-and-index)
+      (define-key map (kbd "C-c C-u") 'mu4e-update-mail-and-index)
+      map)))
 
 (defvar mu4e-compose-mode-abbrev-table nil)
 (define-derived-mode mu4e-compose-mode message-mode "mu4e:compose"
@@ -267,14 +280,13 @@ appear on disk."
 \\{message-mode-map}."
   (progn
     (use-local-map mu4e-compose-mode-map)
-    (set (make-local-variable 'message-signature)
-      mu4e-compose-signature)
+    (set (make-local-variable 'message-signature) mu4e-compose-signature)
+    ;; set this to allow mu4e to work when gnus-agent is unplugged in gnus
+    (set (make-local-variable 'message-send-mail-real-function) nil)
     (make-local-variable 'message-default-charset)
     ;; if the default charset is not set, use UTF-8
     (unless message-default-charset
       (setq message-default-charset 'utf-8))
-    ;; make completion case-insensitive
-    (set (make-local-variable 'completion-ignore-case) t)
     ;; make sure mu4e is started in the background (ie. we don't want to error
     ;; out when sending the message; better to do it now if there's a problem)
     (mu4e~start) ;; start mu4e in background, if needed
@@ -285,13 +297,9 @@ appear on disk."
     (setq default-directory (expand-file-name "~/"))
 
     ;; offer completion for e-mail addresses
-    (when (and mu4e-compose-complete-addresses
-	    (boundp 'completion-at-point-functions))
+    (when mu4e-compose-complete-addresses
       (mu4e~compose-setup-completion))
 
-    (define-key mu4e-compose-mode-map (kbd "C-S-u") 'mu4e-update-mail-and-index)
-    (define-key mu4e-compose-mode-map (kbd "C-c C-u") 'mu4e-update-mail-and-index)
-
     ;; setup the fcc-stuff, if needed
     (add-hook 'message-send-hook
       (lambda () ;; mu4e~compose-save-before-sending
@@ -380,7 +388,7 @@ tempfile)."
   (mu4e~compose-hide-headers)
   ;; switch on the mode
   (mu4e-compose-mode))
- 
+
 (defun mu4e-sent-handler (docid path)
   "Handler function, called with DOCID and PATH for the just-sent
 message. For Forwarded ('Passed') and Replied messages, try to set
@@ -392,9 +400,17 @@ the appropriate flag at the message forwarded or replied-to."
   ;; this seems a bit hamfisted...
   (dolist (buf (buffer-list))
     (when (and (buffer-file-name buf)
-               (string= (buffer-file-name buf) path)
-               message-kill-buffer-on-exit)
-      (kill-buffer buf)))
+               (string= (buffer-file-name buf) path))
+      (if (and mu4e-compose-in-new-frame (window-system))
+	  (progn
+	    (switch-to-buffer buf)
+	    (when (and (get-buffer-window buf)
+		       (window-frame (get-buffer-window buf)))
+	      (delete-frame (window-frame (get-buffer-window buf)))))
+	)
+      (if message-kill-buffer-on-exit
+	  (kill-buffer buf))
+      ))
   ;; now, try to go back to some previous buffer, in the order
   ;; view->headers->main
   (if (buffer-live-p mu4e~view-buffer)
@@ -465,7 +481,13 @@ for draft messages."
     (if (eq compose-type 'new)
       (mu4e~compose-handler 'new)
       ;; otherwise, we need the doc-id
-      (let ((docid (mu4e-message-field msg :docid)))
+      (let* ((docid (mu4e-message-field msg :docid))
+	;; decrypt (or not), based on `mu4e-decryption-policy'.
+	(decrypt
+	  (and (member 'encrypted (mu4e-message-field msg :flags))
+	    (if (eq mu4e-decryption-policy 'ask)
+	      (yes-or-no-p (mu4e-format "Decrypt message?"))
+	      mu4e-decryption-policy))))
 	;; if there's a visible view window, select that before starting composing
 	;; a new message, so that one will be replaced by the compose window. The
 	;; 10-or-so line headers buffer is not a good place to write it...
@@ -473,7 +495,7 @@ for draft messages."
 	  (when (window-live-p viewwin)
 	    (select-window viewwin)))
 	;; talk to the backend
-	(mu4e~proc-compose compose-type docid)))))
+	(mu4e~proc-compose compose-type decrypt docid)))))
 
 (defun mu4e-compose-reply ()
   "Compose a reply for the message at point in the headers buffer."
@@ -497,29 +519,6 @@ draft message."
   (interactive)
   (mu4e-compose 'new))
 
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; address completion; inspired by org-contacts.el
-(defun mu4e~compose-complete-contact (&optional start)
-  "Complete the text at START with a contact.
-Ie. either 'name <email>' or 'email')."
-  (interactive)
-  (let ((mail-abbrev-mode-regexp mu4e~compose-address-fields-regexp)
-	 (eoh ;; end-of-headers
-	   (save-excursion
-	     (goto-char (point-min))
-	     (search-forward-regexp mail-header-separator nil t))))
-    ;; try to complete only when we're in the headers area,
-    ;; looking  at an address field.
-    (when (and eoh (> eoh (point)) (mail-abbrev-in-expansion-header-p))
-      (let* ((end (point))
-	      (start
-		(or start
-		  (save-excursion
-		    (re-search-backward "\\(\\`\\|[\n:,]\\)[ \t]*")
-		    (goto-char (match-end 0))
-		    (point)))))
-	(list start end mu4e~contacts-for-completion)))))
 
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -601,7 +600,7 @@ beginning of the buffer."
   (let ((old-position (point)))
     (message-goto-body)
     (when (equal (point) old-position)
-      (beginning-of-buffer))))
+      (goto-char (point-min)))))
 
 (define-key mu4e-compose-mode-map
   (vector 'remap 'beginning-of-buffer) 'mu4e-compose-goto-top)
@@ -613,11 +612,11 @@ end of the buffer."
   (interactive)
   (let ((old-position (point))
         (message-position (save-excursion (message-goto-body) (point))))
-    (end-of-buffer)
+    (goto-char (point-max))
     (when (re-search-backward message-signature-separator message-position t)
-      (previous-line))
+      (forward-line -1))
     (when (equal (point) old-position)
-      (end-of-buffer))))
+      (goto-char (point-max)))))
 
 (define-key mu4e-compose-mode-map
   (vector 'remap 'end-of-buffer) 'mu4e-compose-goto-bottom)
